@@ -15,12 +15,12 @@ pgFormatter::CLI - Implementation of command line program to format SQL queries.
 
 =head1 VERSION
 
-Version 4.0
+Version 4.2
 
 =cut
 
 # Version of pgFormatter
-our $VERSION = '4.0';
+our $VERSION = '4.2';
 
 use autodie;
 use pgFormatter::Beautify;
@@ -88,6 +88,7 @@ sub beautify {
     $args{ 'spaces' }       = $self->{ 'cfg' }->{ 'spaces' };
     $args{ 'uc_keywords' }  = $self->{ 'cfg' }->{ 'keyword-case' };
     $args{ 'uc_functions' } = $self->{ 'cfg' }->{ 'function-case' };
+    $args{ 'uc_types' }     = $self->{ 'cfg' }->{ 'type-case' };
     $args{ 'placeholder' }  = $self->{ 'cfg' }->{ 'placeholder' };
     $args{ 'separator' }    = $self->{ 'cfg' }->{ 'separator' };
     $args{ 'comma' }        = $self->{ 'cfg' }->{ 'comma' };
@@ -97,6 +98,8 @@ sub beautify {
     $args{ 'format_type' }  = $self->{ 'cfg' }->{ 'format-type' };
     $args{ 'wrap_limit' }   = $self->{ 'cfg' }->{ 'wrap-limit' };
     $args{ 'wrap_after' }   = $self->{ 'cfg' }->{ 'wrap-after' };
+    $args{ 'space' }        = $self->{ 'cfg' }->{ 'space' };
+    $args{ 'no_grouping' }  = $self->{ 'cfg' }->{ 'nogrouping' };
 
     if ($self->{ 'query' } && ($args{ 'maxlength' } && length($self->{ 'query' }) > $args{ 'maxlength' })) {
         $self->{ 'query' } = substr($self->{ 'query' }, 0, $args{ 'maxlength' })
@@ -124,7 +127,14 @@ Saves beautified query to whatever is output filehandle
 
 sub save_output {
     my $self = shift;
-    my $fh   = delete $self->{ 'output' };
+    my $fh;
+    # Thanks to "autodie" I don't have to check if open() worked.
+    if ( $self->{ 'cfg' }->{ 'output' } ne '-' ) {
+        $self->logmsg( 'DEBUG', 'Formatted SQL queries will be written to stdout' );
+        open $fh, '>', $self->{ 'cfg' }->{ 'output' };
+    } else {  
+    	$fh = \*STDOUT;
+    }
     print $fh $self->{ 'ready' };
     close $fh;
     return;
@@ -187,6 +197,8 @@ Options:
                             unchanged: 0. Values: 0=>unchanged, 1=>lowercase,
                             2=>uppercase, 3=>capitalize.
     -F | --format STR     : output format: text or html. Default: text.
+    -g | --nogrouping     : add a newline between statements in transaction
+                            regroupement. Default is to group statements.
     -h | --help           : show this message and exit.
     -m | --maxlength SIZE : maximum length of a query, it will be cutted above
                             the given size. Default: no truncate.
@@ -196,9 +208,14 @@ Options:
     -s | --spaces size    : change space indent, default 4 spaces.
     -S | --separator STR  : dynamic code separator, default to single quote.
     -t | --format-type    : try another formatting type for some statements.
+    -T | --tabs           : use tabs instead of space characters, when used
+                            spaces is set to 1 whatever is the value set to -s.
     -u | --keyword-case N : Change the case of the reserved keyword. Default is
                             uppercase: 2. Values: 0=>unchanged, 1=>lowercase,
                             2=>uppercase, 3=>capitalize.
+    -U | --type-case N    : Change the case of the data type name. Default is
+                            lowercase: 1. Values: 0=>unchanged, 1=>lowercase,
+                            2=>uppercase, 3=>capitalize.
     -v | --version        : show pg_format version and exit.
     -w | --wrap-limit N   : wrap queries at a certain length.
     -W | --wrap-after N   : number of column after which lists must be wrapped.
@@ -230,7 +247,12 @@ Loads SQL from input file or stdin.
 sub load_sql {
     my $self = shift;
     local $/ = undef;
-    my $fh = delete $self->{ 'input' };
+    my $fh;
+    if ( $self->{ 'cfg' }->{ 'input' } ne '-' ) {
+        open $fh, '<', $self->{ 'cfg' }->{ 'input' };
+    } else {
+        $fh = \*STDIN;
+    }
     $self->{ 'query' } = <$fh>;
     close $fh;
     return;
@@ -253,6 +275,7 @@ sub get_command_line_args {
         'debug|d!',
 	'format|F=s',
         'function-case|f=i',
+        'nogrouping|g!',
         'help|h!',
         'maxlength|m=i',
         'nocomment|n!',
@@ -261,7 +284,9 @@ sub get_command_line_args {
         'separator|S=s',
         'spaces|s=i',
         'format-type|t!',
+        'tabs|T!',
         'keyword-case|u=i',
+        'type-case|U=i',
         'version|v!',
         'wrap-limit|w=i',
         'wrap-after|W=i',
@@ -280,6 +305,7 @@ sub get_command_line_args {
     $cfg{ 'output' }        //= '-';
     $cfg{ 'function-case' } //= 0;
     $cfg{ 'keyword-case' }  //= 2;
+    $cfg{ 'type-case' }     //= 1;
     $cfg{ 'comma' }           = 'end';
     $cfg{ 'format' }        //= 'text';
     $cfg{ 'comma-break' }   //= 0;
@@ -287,6 +313,12 @@ sub get_command_line_args {
     $cfg{ 'format-type' }   //= 0;
     $cfg{ 'wrap-limit' }    //= 0;
     $cfg{ 'wrap-after' }    //= 0;
+    $cfg{ 'space' }         //= ' ';
+
+    if ($cfg{ 'tabs' }) {
+        $cfg{ 'spaces' } = 1;
+        $cfg{ 'space' }  = "\t";
+    }
 
     if (!grep(/^$cfg{ 'format' }$/i, 'text', 'html')) {
         printf 'FATAL: unknow output format: %s%s', $cfg{ 'format' } , "\n";
@@ -310,25 +342,7 @@ sub validate_args {
 
     $self->show_help_and_die( 2, 'function-case can be only one of: 0, 1, 2, or 3.' ) unless $self->{ 'cfg' }->{ 'function-case' } =~ m{\A[0123]\z};
     $self->show_help_and_die( 2, 'keyword-case can be only one of: 0, 1, 2, or 3.' )  unless $self->{ 'cfg' }->{ 'keyword-case' } =~ m{\A[0123]\z};
-
-    # Thanks to "autodie" I don't have to check if open() worked.
-    if ( $self->{ 'cfg' }->{ 'input' } eq '-' ) {
-        $self->{ 'input' } = \*STDIN;
-    }
-    else {
-        open my $fh, '<', $self->{ 'cfg' }->{ 'input' };
-        $self->{ 'input' } = $fh;
-    }
-
-    if ( $self->{ 'cfg' }->{ 'output' } eq '-' ) {
-        $self->logmsg( 'DEBUG', 'Formatted SQL queries will be written to stdout' );
-        $self->{ 'output' } = \*STDOUT;
-    }
-    else {
-        $self->logmsg( 'DEBUG', 'Formatted SQL queries will be written to %s', $self->{ 'cfg' }->{ 'output' } );
-        open my $fh, '>', $self->{ 'cfg' }->{ 'output' };
-        $self->{ 'output' } = $fh;
-    }
+    $self->show_help_and_die( 2, 'type-case can be only one of: 0, 1, 2, or 3.' )  unless $self->{ 'cfg' }->{ 'type-case' } =~ m{\A[0123]\z};
 
     if ($self->{ 'cfg' }->{ 'comma-end' }) {
         $self->{ 'cfg' }->{ 'comma' } = 'end';
@@ -350,7 +364,7 @@ Please report any bugs or feature requests to: https://github.com/darold/pgForma
 
 =head1 COPYRIGHT
 
-Copyright 2012-2019 Gilles Darold. All rights reserved.
+Copyright 2012-2020 Gilles Darold. All rights reserved.
 
 =head1 LICENSE
 
